;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Homing Thwomp, by yoshicookiezeus and edited by dahnamics
;;
;; Description: A thwomp that follows Mario horizontally when on the ceiling, and spawns
;;              Spinies, Green Koopas and Hopping Flames randomly when it hits the ground. 
;;	        It has three hit points, and shell like items hurt it.  It gets faster with 
;;              each hit and explodes like a Bob-Omb when killed.  It also ends the level.
;;
;; NOTE: ONLY use this sprite in one-screen areas with "No vertical or horizontal scroll"
;;       enabled!
;;   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Homing Thwomp, by yoshicookiezeus
;;
;; Description: A thwomp that follows Mario horizontally when on the ceiling, and spawns
;;              Bob-Ombs when it hits the ground. It has three hit points, and only gets
;;              hurt by Bob-Omb explosions. It gets faster with each hit.
::
;; NOTE: ONLY use this sprite in one-screen areas with "No vertical or horizontal scroll"
;;       enabled!
;;   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        
	; symbolic names for RAM addresses (don't change these)
	GetSpriteClippingA = $03B69F
	GetSpriteClippingB	= $03B6E5	
	CheckForContact = $03B72B
	ShowSprContactGfx = $01AB72
	SPRITE_Y_SPEED  = $AA
	SPRITE_X_SPEED  = $B6
	SPRITE_STATE    = $C2
	SPRITE_X_POS    = $E4
	SPRITE_Y_POS    = $D8
	ORIG_Y_POS      = $151C
	FREEZE_TIMER    = $1540
	SPR_OBJ_STATUS  = $1588
	H_OFFSCREEN     = $15A0
	V_OFFSCREEN     = $186C
	STATE_TIMER 	= $163E
	RAM_ThrowTimer 	= $1504	
	Invuln_Timer 	= $1564		; The RAM address for Magica's stun duration
	HIT_POINTS = $05        	; being stomped by Mario
	EXTRA_BITS = $7FAB10

	; definitions of bits (don't change these)
	IS_ON_GROUND    = $04             
			
	; sprite data
	SPRITE_GRAVITY  = $04
	MAX_Y_SPEED     = $3E 

	RISE_SPEED	= $C0
	SOUND_EFFECT    = $09 

	TIME_ON_GROUND	= $A0

X_SPEED			dcb $08,$F8,$08,$F8,$0C,$F4,$10,$F0,$14,$EC
	SPRITE_TO_GEN	= $62
						          
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	dcb "INIT"
	LDA SPRITE_Y_POS,x  
	STA ORIG_Y_POS,x
	LDA $E4,x  
	CLC        
	ADC #$08   
	STA $E4,x  

	TXA
	AND #$03
	ASL A
	ASL A
	ASL A
	ASL A
	ASL A                   
	STA STATE_TIMER,x
	CLC
	ADC #$32
	STA RAM_ThrowTimer,x                 

	RTL
      

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

			dcb "MAIN"			                
			PHB			 
			PHK			 
			PLB			 
			JSR SPRITE_CODE_START   
			PLB			 
			RTL			 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite main code 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

RETURN			RTS  

SPRITE_CODE_START	JSR SUB_GFX

			LDA $14C8,x		; return if sprite status != 8

ALIVE		    	CMP #$08                ; \ if status != 8, return
                    	BNE RETURN              ; /          
			
			LDA $9D			; return if sprites locked
			BNE RETURN    

NO_JUMP			JSR SUB_OFF_SCREEN_X0	; only process sprite while on screen           
			
			JSL $01A7DC		; interact with mario 
			BCC NO_CONTACT          ; return if no contact
			LDA $154C,x             ; \ if sprite invincibility timer > 0 ...
			BNE NO_CONTACT          ; /    ... goto NO_CONTACT
			LDA #$08                ; \ sprite invincibility timer = $08
			STA $154C,x             ; /
			LDA $7D                 ; \  if Mario's y speed < 10 ...
			CMP #$10                ;  }   ... sprite will hurt Mario
			BMI SPRITE_WINS         ; /    

MARIO_WINS		JSR SUB_STOMP_PTS       ; give Mario points
			JSL $01AA33             ; set Mario speed
			JSL $01AB99             ; display contact graphic
			LDA #$28                ; \ sound effect
			STA $1DFC               ; /
;			LDA #$01		    ; \ Set stunned state
;			STA $C2,x               ; /
			LDA #$10		    ; \ Set stunned timer
			STA $1564,x             ; /
			INC $1534,x             ; increment sprite hit counter
			LDA $1534,x             ; \ if sprite hit counter == 3
			CMP #HIT_POINTS         ;  |   
			BNE SMUSH_SPRITE        ; /

			LDA #$04                ; \ write frame to show
			STA $1602,x             ; / 
			LDA #$02                ; \ set sprite status   
			STA $14C8,x             ; /
			STZ $B6,x               ; no x speed
			LDA #$10                ; \ set y speed
			STA $AA,x               ; /
			RTS                     ; return 

SMUSH_SPRITE		LDA #$0C                ; \ time to show semi-squashed sprite = $0C
			STA $1FE2,x             ; /
			RTS                     ; return

SPRITE_WINS		LDA $1497               ; \ if Mario is invincible...
			ORA $187A               ;  }  ... or Mario on yoshi...
			BNE NO_CONTACT          ; /   ... return
			JSR SUB_HORZ_POS        ; \  set new sprite direction
			TYA                     ;  }  
			STA $157C,x             ; /
			JSL $00F5B7             ; hurt Mario

NO_CONTACT          	LDA SPRITE_STATE,x     
			CMP #$01
			BEQ FLYING
			CMP #$02
			BEQ FALLING0
			CMP #$03
			BEQ ATTACK0

;-----------------------------------------------------------------------------------------
; state 0
;-----------------------------------------------------------------------------------------

RISING				

		LDA FREEZE_TIMER,x	;if we're still waiting on the ground, return
		BNE RETURN7

		LDA #$08
		STA $151C,x

		LDA SPRITE_Y_POS,x      ;check if the sprite is in original position
		CMP $151C,x
		BNE RISE

		JSL $01ACF9	; get a random number
		ADC $148E	;
		ADC $7B		; add a little more randomness to the mix
		ADC $13		;

		LDA #$C0		; set the time of flight to C0
		CMP #$80		; if our random number was less than 80...
		BCC STORESTATENUM	; the time of flight is now C0
		LDA #$F0		; set the time of flight to F0
		CMP #$80		; if our random number was 80 or greater...
		BCS STORESTATENUM	; the time of flight is now F0
STORESTATENUM				
		STA STATE_TIMER,x
			
		INC SPRITE_STATE,x	; set state to flying
		RTS			 

RISE		LDY $1528,x
		LDA #RISE_SPEED		;set rising speed and apply it
		STA SPRITE_Y_SPEED,x
		JSL $01801A             
RETURN7		RTS			 

FALLING0	BRA FALLING1
ATTACK0		BRA ATTACK1

;-----------------------------------------------------------------------------------------
; state 1
;-----------------------------------------------------------------------------------------

FLYING		LDA V_OFFSCREEN,x	;fall if offscreen vertically
		BNE SET_FALLING
			
		LDA H_OFFSCREEN,x	;return if offscreen horizontally
		BNE RETURN0
			
		STZ SPRITE_Y_SPEED,x

		LDA $1588,x             ; \ if sprite is in contact with an object...
		AND #$03                ;  |
		BEQ NO_OBJ_CONTACT      ;  |
		LDA $157C,x             ;  | flip the direction status
		EOR #$01                ;  |
		STA $157C,x             ; /

NO_OBJ_CONTACT
		LDA $1534,x             ; \ set x speed based on total HP
		ASL
		CLC
		ADC $157C,x		; and direction
		TAY

		LDA X_SPEED,y           ;  |
		STA $B6,x               ; /

CONTINUE	LDA STATE_TIMER,x
		CMP #$28                ; |   just go to normal walking code
		BCS RETURN0          ; /
                    
		LDA Invuln_Timer,x
		CMP #$00
		BNE RETURN0

		LDA STATE_TIMER,x
		CMP #$21
		BNE NO_JUMP2
		LDA $1570,x
		LSR A
		AND #01
		BEQ NO_JUMP2
		STZ STATE_TIMER,x					; x speed = 0                  

		BRA RETURN0
NO_JUMP2	CMP #$00
		BNE RETURN0

SET_FALLING	STZ SPRITE_X_SPEED,x

		LDA #$10                ; \ sound effect
		STA $1DF9 

		INC SPRITE_STATE,x	;chage state to falling

		LDA #$00     
		STA SPRITE_Y_SPEED,x	;set initial speed

FALLING1	BRA FALLING

RETURN0		JSL $01802A		;apply speed
		RTS	

ATTACK1		BRA ATTACK2		 

;-----------------------------------------------------------------------------------------
; state 2
;-----------------------------------------------------------------------------------------

FALLING		JSL $01801A		;apply speed
			
		LDA SPRITE_Y_SPEED,x	;increase speed if below the max
		CMP #MAX_Y_SPEED
		BCS DONT_INC_SPEED
		ADC #SPRITE_GRAVITY
		STA SPRITE_Y_SPEED,x    
DONT_INC_SPEED
		JSL $019138		;interact with objects
			
		LDA SPR_OBJ_STATUS,x	;return if not on the ground
		AND #IS_ON_GROUND
		BEQ RETURN68
		    
		JSR SUB_HORZ_POS        ; \ always face Mario
		TYA                     ;  |
		STA $157C,x             ; /

		LDA #TIME_ON_GROUND	;set time to stay on ground
		STA FREEZE_TIMER,x  
									
		INC SPRITE_STATE,x	;go to attacking state

ATTACK2		BRA ATTACK

RETURN68	RTS     	 

;-----------------------------------------------------------------------------------------
; state 3
;-----------------------------------------------------------------------------------------

		LDA #$A0
		sTA STATE_TIMER,x
ATTACK

		JSR SUB_9A04		; ?? speed related

		LDY $1528,x			

CONTINUE	LDA STATE_TIMER,x
		CMP #$28                ; |   just go to normal walking code
		BCS RETURN0          ; /
                    
		LDA Invuln_Timer,x
		CMP #$00
		BNE RETURN0

		LDA STATE_TIMER,x
		CMP #$21
		BNE NO_JUMP3
		LDA $1570,x
		LSR A
		AND #01
		BEQ NO_JUMP3
		STZ STATE_TIMER,x					; x speed = 0                  

		BRA RETURN67
NO_JUMP3	CMP #$00
		BNE RETURN67

		JSR SUB_HAMMER_THROW    ; /
		JSR SUB_HAMMER_THROW2    ; /
		JSR SUB_HAMMER_THROW3    ; /

		STZ SPRITE_STATE,x	; go to rising state

RETURN67	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; hammer routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
X_OFFSET	dcb $06,$FA
X_OFFSET2	dcb $00,$FF
X_SPEED_HAMMERa	dcb $40,$C0
Y_SPEED_HAMMERa	dcb $00,$FF
X_SPEED_HAMMERc	dcb $40,$C0
Y_SPEED_HAMMERc	dcb $80,$80
X_SPEED_HAMMERd	dcb $20,$E0
Y_SPEED_HAMMERd	dcb $80,$80

SUB_HAMMER_THROW    		
		JSL $02A9DE             ; \ get an index to an unused sprite slot, return if all slots full
		BMI LABEL6            ; / after: Y has index of sprite being generated

		PHX
		TYX

		LDA #$10                ; \ sound effect
		STA $1DF9 

		LDA #$08
		STA $14C8,x

		LDA #SPRITE_TO_GEN
		STA $7FAB9E,x
		JSL $07F7D2
		JSL $0187A7
		LDA #$08
		STA $7FAB10,x
		PLX

		PHY
		LDA $157C,x
		TAY
		LDA $E4,x               ; \ set xlo position for new sprite
		CLC
		ADC X_OFFSET,y
		PLY
		STA $00E4,y

		PHY
		LDA $157C,x
		TAY
		LDA $14E0,x             ;  | set xhi position for new sprite
		ADC X_OFFSET2,y
		PLY
		STA $14E0,y             ; /

		LDA $D8,x               ; \ set y position for new sprite
		SEC                     ;  | (y position of generator - 1)
		SBC #$0E                ;  |
		STA $00D8,y             ;  |
		LDA $14D4,x             ;  |
		SBC #$00                ;  |
		STA $14D4,y             ; /

		PHX                     ; \ before: X must have index of sprite being generated
		TYX                     ;  | routine clears *all* old sprite values...
		JSL $07F7D2             ;  | ...and loads in new values for the 6 main sprite tables
		JSL $0187A7             ;  get table values for custom sprite
		LDA #$88
		STA EXTRA_BITS,x
		PLX                       ; / 
                    
		PHX                     ; \  set x speed of fire ball
		LDA $157C,x             ;  |
		TAX                     ;  |
		LDA X_SPEED_HAMMERa,x            ;  |
		STA $B6,y             ; /
		PLX 

LABEL6              RTS        

SUB_HAMMER_THROW2    
		JSL $02A9DE             ; \ get an index to an unused sprite slot, return if all slots full
		BMI LABEL6c            ; / after: Y has index of sprite being generated

		PHX
		TYX

		LDA #$08
		STA $14C8,x

		LDA #SPRITE_TO_GEN
		STA $7FAB9E,x
		JSL $07F7D2
		JSL $0187A7
		LDA #$08
		STA $7FAB10,x
		PLX

		PHY
		LDA $157C,x
		TAY
		LDA $E4,x               ; \ set xlo position for new sprite
		CLC
		ADC X_OFFSET,y
		PLY
		STA $00E4,y

		PHY
		LDA $157C,x
		TAY
		LDA $14E0,x             ;  | set xhi position for new sprite
		ADC X_OFFSET2,y
		PLY
		STA $14E0,y             ; /

		LDA $D8,x               ; \ set y position for new sprite
		SEC                     ;  | (y position of generator - 1)
		SBC #$0E                ;  |
		STA $00D8,y             ;  |
		LDA $14D4,x             ;  |
		SBC #$00                ;  |
		STA $14D4,y             ; /

		PHX                     ; \ before: X must have index of sprite being generated
		TYX                     ;  | routine clears *all* old sprite values...
		JSL $07F7D2             ;  | ...and loads in new values for the 6 main sprite tables
		JSL $0187A7             ;  get table values for custom sprite
		LDA #$88
		STA EXTRA_BITS,x
		PLX                       ; / 
                    
		PHX                     ; \ set y speed of fire ball
		LDA $151C,x             ;  |
		TAX                     ;  |
		LDA Y_SPEED_HAMMERc,x            ;  |
		STA $AA,y             ;  |
		PLX                     ; /     
                    
		PHX                     ; \  set x speed of fire ball
		LDA $157C,x             ;  |
		TAX                     ;  |
		LDA X_SPEED_HAMMERc,x            ;  |
		STA $B6,y             ; /
		PLX 

LABEL6c              RTS       

SUB_HAMMER_THROW3    
		JSL $02A9DE             ; \ get an index to an unused sprite slot, return if all slots full
		BMI LABEL6d            ; / after: Y has index of sprite being generated

		PHX
		TYX

		LDA #$08
		STA $14C8,x

		LDA #SPRITE_TO_GEN
		STA $7FAB9E,x
		JSL $07F7D2
		JSL $0187A7
		LDA #$08
		STA $7FAB10,x
		PLX

		PHY
		LDA $157C,x
		TAY
		LDA $E4,x               ; \ set xlo position for new sprite
		CLC
		ADC X_OFFSET,y
		PLY
		STA $00E4,y

		PHY
		LDA $157C,x
		TAY
		LDA $14E0,x             ;  | set xhi position for new sprite
		ADC X_OFFSET2,y
		PLY
		STA $14E0,y             ; /

		LDA $D8,x               ; \ set y position for new sprite
		SEC                     ;  | (y position of generator - 1)
		SBC #$0E                ;  |
		STA $00D8,y             ;  |
		LDA $14D4,x             ;  |
		SBC #$00                ;  |
		STA $14D4,y             ; /

		PHX                     ; \ before: X must have index of sprite being generated
		TYX                     ;  | routine clears *all* old sprite values...
		JSL $07F7D2             ;  | ...and loads in new values for the 6 main sprite tables
		JSL $0187A7             ;  get table values for custom sprite
		LDA #$88
		STA EXTRA_BITS,x
		PLX                       ; / 
                    
		PHX                     ; \ set y speed of fire ball
		LDA $151C,x             ;  |
		TAX                     ;  |
		LDA Y_SPEED_HAMMERd,x            ;  |
		STA $AA,y             ;  |
		PLX                     ; /     
                    
		PHX                     ; \  set x speed of fire ball
		LDA $157C,x             ;  |
		TAX                     ;  |
		LDA X_SPEED_HAMMERd,x            ;  |
		STA $B6,y             ; /
		PLX 

LABEL6d              RTS       



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PROPERTIES: dcb $71,$31

;The tables must now have 16 bytes.
;THE LAST 8 ARE ONLY CREATED BECAUSE OF XDISP.

;0-4 BYTE   - FRAME 1 RIGHT
;4-8 BYTE   - FRAME 2 RIGHT
;8-12 BYTE  - FRAME 1 LEFT
;12-16 BYTE - FRAME 2 LEFT

TILEMAP:
	dcb $08,$0A,$28,$2A ; FLYING 1 	;\ RIGHT
	dcb $40,$42,$60,$62 ; FLYING 2 	;/ RIGHT
	dcb $00,$02,$20,$22 ; ATTACK 1 	;\ RIGHT
	dcb $04,$06,$24,$26 ; ATTACK 2 	;/ RIGHT

	dcb $08,$0A,$28,$2A ; FLYING 1 	;\ LEFT
	dcb $40,$42,$60,$62 ; FLYING 2 	;/ LEFT
	dcb $00,$02,$20,$22 ; ATTACK 1 	;\ RIGHT
	dcb $04,$06,$24,$26 ; ATTACK 2 	;/ RIGHT

YDISP: 	
	dcb $F0,$F0,$00,$00 ; FLYING 1 	;\ RIGHT
	dcb $F0,$F0,$00,$00 ; FLYING 2 	;/ RIGHT
	dcb $F0,$F0,$00,$00 ; ATTACK 1 	;\ RIGHT
	dcb $F0,$F0,$00,$00 ; ATTACK 2 	;/ RIGHT

	dcb $F0,$F0,$00,$00 ; FLYING 1 	;\ LEFT
	dcb $F0,$F0,$00,$00 ; FLYING 2 	;/ LEFT
	dcb $F0,$F0,$00,$00 ; ATTACK 1 	;\ RIGHT
	dcb $F0,$F0,$00,$00 ; ATTACK 2 	;/ RIGHT

XDISP: 	
	dcb $00,$10,$00,$10 ; FLYING 1 	;\ RIGHT
	dcb $00,$10,$00,$10 ; FLYING 2 	;/ RIGHT
	dcb $00,$10,$00,$10 ; ATTACK 1 	;\ RIGHT
	dcb $00,$10,$00,$10 ; ATTACK 2 	;/ RIGHT

	dcb $10,$00,$10,$00 ; FLYING 1 	;\ LEFT
	dcb $10,$00,$10,$00 ; FLYING 2 	;/ LEFT
	dcb $10,$00,$10,$00 ; ATTACK 1 	;\ LEFT
	dcb $10,$00,$10,$00 ; ATTACK 2 	;/ LEFT


SUB_GFX:
	JSR GET_DRAW_INFO

	LDA $14			;\ Frame counter ..
	LSR			; |
	LSR			; | Add in frame animation rate; More LSRs for slower animation.
	LSR			; |
	LSR			; | Add in frame animation rate; More LSRs for slower animation.
	AND #$01		; | 01 means we animate between 2 frames (00 and 01).
	ASL A 			; | 
	ASL A			; | ASL x2 (0-4) makes it switch between the first byte and fifth byte,
	STA $03 		;/ i.e. first animation and second animation. The result is stored into $03.

NOT_DEAD:
	LDA SPR_OBJ_STATUS,x	;return if not on the ground
	CMP #$04				; |
	BNE DONE_WALKING				; |
	LDA $03					; |
	CLC						; |
	ADC #$08				; | ...use stun frames.
	STA $03					; /
	BRA DONE_WALKING

DONE_WALKING:
	LDA $157C,x
	STA $02			; Store direction to $02 for use with property routine later.
	BNE NoAdd
	LDA $03			;\
	CLC			; | If sprite faces left ..
	ADC #$10		; | Adding 16 more bytes to the table.
	STA $03			;/ So we can invert XDISP to not mess up the sprite's appearance.
NoAdd:
	PHX			;\ Push sprite index ..
	LDX #$03		;/ And load X with number of tiles to loop through.
Loop:
	PHX			; Push number of tiles to loop through.
	TXA			;\
	ORA $03			;/ Transfer it to X and add in the "left displacement" if necessary.
	TAX			;\ Get it back into X for an index.

	LDA $00			;\
	CLC			; | Apply X displacement of the sprite.
	ADC XDISP,x		; |
	STA $0300,y		;/ 

	LDA $01			;\
	CLC			; | Y displacement is added for the Y position, so one tile is higher than the other.
	ADC YDISP,x		; | Otherwise, both tiles would have been drawn to the same position!
	STA $0301,y		; | If X is 00, i.e. first tile, then load the first value from the table and apply that
				;/ as the displacement. For the second tile, F0 is added to make it higher than the first.

	LDA TILEMAP,x
	STA $0302,y

	PHX			; Push number of times to go through loop + "left" displacement if necessary.
	LDX $02			;\
	LDA PROPERTIES,x	; | Set properties based on direction.
	STA $0303,y		;/
	PLX			; Pull number of times to go through loop.

	INY			;\
	INY			; | The OAM is 8x8, but our sprite is 16x16 ..
	INY			; | So increment it 4 times.
	INY			;/
		
	PLX			; Pull current tile back.
	DEX			; After drawing this tile, decrease number of tiles to go through loop. If the second tile
				; is drawn, then loop again to draw the first tile.

	BPL Loop		; Loop until X becomes negative (FF).
	
	PLX			; Pull back the sprite index! We pushed it at the beginning of the routine.

	LDY #$02		; Y ends with the tile size .. 02 means it's 16x16
	LDA #$03		; A -> number of tiles drawn - 1.
				; I drew 2 tiles, so 2-1 = 1. A = 01.

	JSL $01B7B3		; Call the routine that draws the sprite.
	RTS			; Never forget this!       

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; speed related
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			
SUB_9A04            LDA SPR_OBJ_STATUS,x
			BMI THWOMP_1
			LDA #$00                
			LDY $15B8,x
			BEQ THWOMP_2
THWOMP_1            LDA #$18
THWOMP_2            STA SPRITE_Y_SPEED,x  
			RTS		

;###############################################################################################
;###############################################################################################
;###############################################################################################
; ROUTINES FROM THE LIBRARY ARE PASTED BELOW
; You should never have to modify this code
;###############################################################################################
;###############################################################################################
;###############################################################################################

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; points routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


STAR_SOUNDS         dcb $00,$13,$14,$15,$16,$17,$18,$19

SUB_STOMP_PTS       PHY                     ; 
                    LDA $1697               ; \
                    CLC                     ;  | 
                    ADC $1626,x             ; / some enemies give higher pts/1ups quicker??
                    INC $1697               ; increase consecutive enemies stomped
                    TAY                     ;
                    INY                     ;
                    CPY #$08                ; \ if consecutive enemies stomped >= 8 ...
                    BCS NO_SOUND            ; /    ... don't play sound 
                    LDA STAR_SOUNDS,y       ; \ play sound effect
                    STA $1DF9               ; /   
NO_SOUND            TYA                     ; \
                    CMP #$08                ;  | if consecutive enemies stomped >= 8, reset to 8
                    BCC NO_RESET            ;  |
                    LDA #$08                ; /
NO_RESET            JSL $02ACE5             ; give Mario points
                    PLY                     ;
                    RTS                     ; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B817 - horizontal mario/sprite check - shared
; Y = 1 if mario left of sprite??
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B817                ; Y = 1 if contact

SUB_GET_DIR         LDY #$00                ;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
                    LDA $94                 ;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
                    SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
                    SBC $E4,x               ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
                    STA $0F                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
                    LDA $95                 ;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
                    SBC $14E0,x             ;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
                    BPL LABEL16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
                    INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
LABEL16             RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_HORZ_POS
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B817
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_HORZ_POS		LDY #$00				;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
					LDA $94					;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
					SEC			 ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
					SBC $E4,x				;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
					STA $0F					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
					LDA $95					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
					SBC $14E0,x				;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
					BPL SPR_L16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
					INY			 ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
SPR_L16				RTS			 ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
			
SPR_T12             dcb $40,$B0
SPR_T13             dcb $01,$FF
SPR_T14             dcb $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
		            dcb $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15             dcb $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
					dcb $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1   LDA #$02                ; \ entry point of routine determines value of $03
			BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2   LDA #$04                ;  | 
			BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3   LDA #$06                ;  |
			BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4   LDA #$08                ;  |
			BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5   LDA #$0A                ;  |
			BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6   LDA #$0C                ;  |
			BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7   LDA #$0E                ;  |
STORE_03			STA $03					;  |            
					BRA START_SUB			;  |
SUB_OFF_SCREEN_X0   STZ $03					; /

START_SUB           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
			BEQ RETURN_35           ; /
			LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
			AND #$01                ; |
			BNE VERTICAL_LEVEL      ; /     
			LDA SPRITE_Y_POS,x               ; \
			CLC			 ; | 
			ADC #$50                ; | if the sprite has gone off the bottom of the level...
			LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
			ADC #$00                ; | 
			CMP #$02                ; | 
			BPL ERASE_SPRITE        ; /    ...erase the sprite
			LDA $167A,x             ; \ if "process offscreen" flag is set, return
			AND #$04                ; |
			BNE RETURN_35           ; /
			LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
			AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
			ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
			STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
			TAY			 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
			LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
			CLC			 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
			ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
			ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
			CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
			PHP			 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
			LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
			LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
			ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
			PLP			 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
			SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
			STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
			LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
			BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
			EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
			STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31             LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
			BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
			CMP #$08                ; |
			BCC KILL_SPRITE         ; /    
			LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
			CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
			BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
			LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
			STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE         STZ $14C8,x             ; erase sprite
RETURN_35           RTS			 ; return

VERTICAL_LEVEL      LDA $167A,x             ; \ if "process offscreen" flag is set, return
			AND #$04                ; |
			BNE RETURN_35           ; /
			LDA $13                 ; \
			LSR A                   ; | 
			BCS RETURN_35           ; /
			LDA $E4,x               ; \ 
			CMP #$00                ;  | if the sprite has gone off the side of the level...
			LDA $14E0,x             ;  |
			SBC #$00                ;  |
			CMP #$02                ;  |
			BCS ERASE_SPRITE        ; /  ...erase the sprite
			LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
			LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
			AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
			STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
			TAY			 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
			LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
			CLC			 ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
			ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
			ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
			CMP SPRITE_Y_POS,x      ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
			PHP			 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
			LDA.W $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
			LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
			ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
			PLP			 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
			SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
			STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
			LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
			BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
			EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
			STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38             LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
			BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
			BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN   LDA H_OFFSCREEN,x       ; \ if sprite is on screen, accumulator = 0 
			ORA V_OFFSCREEN,x       ; |  
			RTS			 ; / return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1              dcb $0C,$1C
SPR_T2              dcb $01,$02

GET_DRAW_INFO       STZ V_OFFSCREEN,x       ; reset sprite offscreen flag, vertical
			STZ H_OFFSCREEN,x       ; reset sprite offscreen flag, horizontal
			LDA $E4,x               ; \
			CMP $1A                 ;  | set horizontal offscreen if necessary
			LDA $14E0,x             ;  |
			SBC $1B                 ;  |
			BEQ ON_SCREEN_X         ;  |
			INC H_OFFSCREEN,x       ; /

ON_SCREEN_X         LDA $14E0,x             ; \
			XBA			 ;  |
			LDA $E4,x               ;  |
			REP #$20                ;  |
			SEC			 ;  |
			SBC $1A                 ;  | mark sprite invalid if far enough off screen
			CLC			 ;  |
			ADC.W #$0040            ;  |
			CMP.W #$0180            ;  |
			SEP #$20                ;  |
			ROL A                   ;  |
			AND #$01                ;  |
			STA $15C4,x             ; / 
			BNE INVALID             ; 
			
			LDY #$00                ; \ set up loop:
			LDA $1662,x             ;  | 
			AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
			BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
			INY			 ; / 
ON_SCREEN_LOOP      LDA SPRITE_Y_POS,x               ; \ 
			CLC			 ;  | set vertical offscreen if necessary
			ADC SPR_T1,y            ;  |
			PHP			 ;  |
			CMP $1C                 ;  | (vert screen boundry)
			ROL $00                 ;  |
			PLP			 ;  |
			LDA $14D4,x             ;  | 
			ADC #$00                ;  |
			LSR $00                 ;  |
			SBC $1D                 ;  |
			BEQ ON_SCREEN_Y         ;  |
			LDA V_OFFSCREEN,x       ;  | (vert offscreen)
			ORA SPR_T2,y            ;  |
			STA V_OFFSCREEN,x       ;  |
ON_SCREEN_Y         DEY			 ;  |
			BPL ON_SCREEN_LOOP      ; /

			LDY $15EA,x             ; get offset to sprite OAM
			LDA $E4,x               ; \ 
			SEC			 ;  | 
			SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
			STA $00                 ; / 
			LDA SPRITE_Y_POS,x               ; \ 
			SEC			 ;  | 
			SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
			STA $01                 ; / 
			RTS			 ; return

INVALID             PLA			 ; \ return from *main gfx routine* subroutine...
			PLA			 ;  |    ...(not just this subroutine)
			RTS			 ; /
			